package ru.dbotthepony.kstarbound.defs.item.api

import ru.dbotthepony.kommons.util.Either
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.defs.AssetPath
import ru.dbotthepony.kstarbound.defs.IThingWithDescription
import ru.dbotthepony.kstarbound.defs.item.IInventoryIcon
import ru.dbotthepony.kstarbound.defs.item.ItemRarity
import ru.dbotthepony.kstarbound.defs.item.impl.ItemDefinition
import ru.dbotthepony.kstarbound.json.builder.JsonImplementation

@JsonImplementation(ItemDefinition::class)
interface IItemDefinition : IThingWithDescription {
	/**
	 * Внутреннее имя предмета (ID).
	 * Не путать с именем предмета!
	 *
	 * @see shortdescription
	 * @see description
	 */
	val itemName: String

	/**
	 * Цена в пикселях
	 */
	val price: Long

	/**
	 * Редкость предмета
	 */
	val rarity: ItemRarity

	/**
	 * Категория предмета, определяет, в какую вкладку инвентаря оно попадает
	 */
	val category: String?

	/**
	 * Иконка в инвентаре, относительный и абсолютный пути
	 */
	val inventoryIcon: Either<out IInventoryIcon, out List<IInventoryIcon>>?

	/**
	 * Теги предмета
	 */
	val itemTags: List<String>

	/**
	 * При подборе предмета мгновенно заставляет игрока изучить эти рецепты крафта
	 */
	val learnBlueprintsOnPickup: List<Registry.Ref<IItemDefinition>>

	/**
	 * Максимальное количество предмета в стопке
	 */
	val maxStack: Long

	/**
	 * snip
	 */
	val eventCategory: String?

	/**
	 * Топливо корабля
	 */
	val fuelAmount: Long

	/**
	 * Заставляет предмет "использовать" сразу же при подборе
	 */
	val consumeOnPickup: Boolean

	/**
	 * Запускает следующие квест(ы) при подборе
	 */
	val pickupQuestTemplates: List<String>

	/**
	 * тип tooltip'а
	 *
	 * ссылается на конфиг окон
	 */
	val tooltipKind: String

	/**
	 * Занимает ли предмет обе руки
	 */
	val twoHanded: Boolean

	/**
	 * Заставляет SAIL/прочих болтать при подборе предмета в первый раз
	 */
	val radioMessagesOnPickup: List<String>

	/**
	 * Звуки при поднятии "малого" количества предметов. Не имеет никакого смысла без [smallStackLimit]
	 */
	val pickupSoundsSmall: List<String>?

	/**
	 * Звуки при поднятии "среднего" количества предметов. Не имеет никакого смысла без [mediumStackLimit]
	 */
	val pickupSoundsMedium: List<String>?

	/**
	 * Звуки при поднятии "большого" количества предметов. Не имеет никакого смысла без [smallStackLimit] и без [mediumStackLimit]
	 */
	val pickupSoundsLarge: List<String>?

	/**
	 * Количество предметов ниже или равному данному значению проиграет звук [pickupSoundsSmall]
	 */
	val smallStackLimit: Long?

	/**
	 * Количество предметов ниже или равному данному значению (но не меньше [smallStackLimit]) проиграет звук [pickupSoundsMedium]
	 */
	val mediumStackLimit: Long?

	/**
	 * Lua script path for item config builder
	 */
	val builder: AssetPath?

	/**
	 * Тип предмета, используется Lua скриптами
	 */
	val itemType: String
		get() = "item"
}
